﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Leadtools;
using Leadtools.Codecs;
using Leadtools.Drawing;
using Leadtools.Controls;
using Leadtools.Forms;
using Leadtools.Forms.Ocr;
using Leadtools.Forms.DocumentWriters;

namespace OCRTutorial3
{
    public partial class Form1 : Form
    {
        // Image viewer
        private ImageViewer _imageViewer;
        // OCR 引擎
        private IOcrEngine _ocrEngine;
        // OCR 页面
        private IOcrPage _ocrPage;
        // 识别结果 (带有字符位置信息)
        private IOcrZoneCharacters _zoneCharacters;

        public Form1()
        {
            string licenseFilePath = @"D:\Program Files\LEADTOOLS 19\Common\License\LEADTOOLS.LIC";
            string developerKey = "*****";
            RasterSupport.SetLicense(licenseFilePath, developerKey);
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            // 将Image Viewer添加到Form1中
            _imageViewer = new ImageViewer();
            _imageViewer.Dock = DockStyle.Fill;
            Controls.Add(_imageViewer);
            _imageViewer.BringToFront();
            // 显示图片原始大小
            _imageViewer.UseDpi = true;

            // 添加鼠标左键拖动图片
            ImageViewerPanZoomInteractiveMode panZoomMode = new ImageViewerPanZoomInteractiveMode();
            panZoomMode.MouseButtons = MouseButtons.Left;
            _imageViewer.InteractiveModes.Add(panZoomMode);
            // 为右键添加括选功能
            ImageViewerRubberBandInteractiveMode rubbBandMode = new ImageViewerRubberBandInteractiveMode();
            rubbBandMode.MouseButtons = MouseButtons.Right;
            // 追加一个事件到括选结束，这样能够在结束时识别括选内容
            rubbBandMode.RubberBandCompleted += rubbBandMode_RubberBandCompleted;
            _imageViewer.InteractiveModes.Add(rubbBandMode);

            // 初始化OCR引擎
            _ocrEngine = OcrEngineManager.CreateEngine(OcrEngineType.Advantage, false);
            // 启动引擎
            _ocrEngine.Startup(null, null, null, @"D:\Program Files\LEADTOOLS 19\Bin\Common\OcrAdvantageRuntime");
            // 启用中文、英文识别
            _ocrEngine.LanguageManager.EnableLanguages(new string[] { "zh-Hans", "en" });

            // 使用示例图片创建一个页面
            string fileName = @"..\..\ocr1.jpg";
            RasterImage rasterImage = _ocrEngine.RasterCodecsInstance.Load(fileName, 1);
            _ocrPage = _ocrEngine.CreatePage(rasterImage, OcrImageSharingMode.AutoDispose);
            // 在Viewer中显示这个图片
            _imageViewer.Image = _ocrPage.GetRasterImage();
            Text = "鼠标左键拖拽图片, CTRL+左键可以缩放, 右键选择识别区域";

            // 添加PostRender事件，用来绘制识别括选区域
            _imageViewer.PostRender += _imageViewer_PostRender;

            base.OnLoad(e);
        }
        protected override void OnFormClosed(FormClosedEventArgs e)
        {
            // 销毁页面
            _ocrPage.Dispose();

            // 释放引擎
            _ocrEngine.Dispose();

            base.OnFormClosed(e);
        }
        private void rubbBandMode_RubberBandCompleted(object sender, ImageViewerRubberBandEventArgs e)
        {
            // 使用已经绘制好的区域

            // 先移除所有识别区域
            _ocrPage.Zones.Clear();

            // 添加一个新的区域， 绘制的矩形区域是控件的坐标，我们需要将这个坐标转换为图片坐标
            LeadRect bounds = LeadRect.FromLTRB(e.Points[0].X, e.Points[0].Y, e.Points[1].X, e.Points[1].Y);
            bounds = _imageViewer.ConvertRect(null, ImageViewerCoordinateType.Control, ImageViewerCoordinateType.Image, bounds);

            // 使用这个坐标来创建一个识别区域
            OcrZone ocrZone = new OcrZone();
            ocrZone.ZoneType = OcrZoneType.Text;
            ocrZone.Bounds = new LogicalRectangle(bounds);
            _ocrPage.Zones.Add(ocrZone);

            // 进行识别，获取文字
            _ocrPage.Recognize(null);

            // 取得识别结果
            IOcrPageCharacters pageCharacters = _ocrPage.GetRecognizedCharacters();
            _zoneCharacters = pageCharacters[0];

            // 刷新Viewer来绘制识别字符
            _imageViewer.Refresh();

            // 使用Messagebox来显示识别结果
            string text = _ocrPage.GetText(0);
            if (string.IsNullOrEmpty(text))
                text = "[无文字]";
            MessageBox.Show(this, text);
        }

        private void _imageViewer_PostRender(object sender, ImageViewerRenderEventArgs e)
        {
            // 绘制识别结果，如果有的话
            if (_zoneCharacters == null)
                return;

            Graphics graphics = e.PaintEventArgs.Graphics;

            using (Brush characterBrush = new SolidBrush(Color.FromArgb(128, Color.Black)))
            {
                foreach (OcrCharacter character in _zoneCharacters)
                {
                    // 取得字符位置
                    LogicalRectangle characterBounds = character.Bounds;
                    // 字符位置是一个逻辑区域, 有可能在单位像素外，将它转换为像素
                    LeadRect bounds = characterBounds.ToRectangle(_ocrPage.DpiX, _ocrPage.DpiY);
                    // 转换这个位置到Viewer上
                    // 注意，这个Demo并没有旋转图片所以你需要使用四个点来定位
                    bounds = _imageViewer.ConvertRect(null, ImageViewerCoordinateType.Image, ImageViewerCoordinateType.Control, bounds);

                    // 高亮字符区域
                    graphics.FillRectangle(characterBrush, bounds.X, bounds.Y, bounds.Width, bounds.Height);
                    graphics.DrawRectangle(Pens.Black, bounds.X, bounds.Y, bounds.Width - 1, bounds.Height - 1);

                    // 最后绘制字符
                    graphics.DrawString(new string(new char[] { character.Code }), this.Font, Brushes.White, bounds.X, bounds.Y);
                }
            }
        }
    }
}
